import asyncio
import os
import traceback

import aiohttp
from aiolimiter import AsyncLimiter
from dotenv import load_dotenv

load_dotenv()
RATE_LIMIT_PER_SECOND = int(os.getenv("RATE_LIMIT_PER_SECOND"))
limiter = AsyncLimiter(
    max_rate=float(os.getenv("ASYNC_LIMITER_MAX_RATE")),
    time_period=float(os.getenv("ASYNC_LIMITER_TIME_PERIOD")),
)


async def fetch_json(session, url):
    async with session.get(url) as response:
        return await response.json()


async def fetch_text(session, url):
    async with session.get(url) as response:
        return await response.text()


async def get(urls: list[str], expected_response: str):
    async with aiohttp.ClientSession() as session:
        i = 0
        total_results = []
        while i < len(urls):
            async with limiter:
                coroutines = []

                if i + RATE_LIMIT_PER_SECOND < len(urls):
                    _urls = urls[i: i + RATE_LIMIT_PER_SECOND]
                else:
                    _urls = urls[i:]

                if expected_response == "csv":
                    func = fetch_text
                else:
                    func = fetch_json

                for url in _urls:
                    coroutines.append(func(session=session, url=url))
                results = await asyncio.gather(*coroutines, return_exceptions=True)
            total_results += results
            i += RATE_LIMIT_PER_SECOND

    err = None
    for result, coro in zip(total_results, coroutines):
        if isinstance(result, Exception):
            err = result
            print(f"{coro.__name__} failed:")
            traceback.print_exception(type(err), err, err.__traceback__)

    if err:
        raise RuntimeError("One or more scripts failed.")

    return total_results



import asyncio
import platform
from csv import DictReader

import unidecode

from . import request

if platform.system() == "Windows":
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())


def get_runemetrics(players: list[str]) -> str:
    """
    Takes a list of players and accesses the runemetrics API endpoint.  The results are aggregated and return as json.
    :param players:
    :return:
    """
    urls = [
        f"https://apps.runescape.com/runemetrics/profile/profile?user={player}&activities=20"
        for player in players
    ]
    return asyncio.run(request.get(urls=urls, expected_response="json"))


def get_clan_data(clans: list[str]) -> str:
    urls = [
        f"https://services.runescape.com/m=clan-hiscores/members_lite.ws?clanName={clan}"
        for clan in clans
    ]
    return asyncio.run(request.get(urls=urls, expected_response="csv"))


def get_clan_members(clans: list[str]):
    clan_members = get_clan_data(clans=clans)

    list_of_members = []
    for clan in clan_members:
        d_r = DictReader(unidecode.unidecode(clan).splitlines())
        list_of_members.append([elem["Clanmate"] for elem in list(d_r)])

    return list_of_members



import sys

import setuptools

__version__ = "0.2.0"

setuptools.setup(
    name="scapers-api",
    version=__version__,
    author="Jarod Daming",
    author_email="jmdaming@gmail.com",
    description="Allows http access to Runescape API Endpoints",
    url="https://gitlab.com/maximized/scapers_api",
    project_url={"Bug Tracker": "https://gitlab.com/maximized/scapers_api/-/issues"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.9",
    install_requires=[
        "aiohttp~=3.8.3",
        "aiolimiter~=1.0.0",
        "python-semantic-release==7.32.0",
        "python-dotenv~=0.21.0",
        "setuptools==65.4.0",
    ],
)

try:
    from semantic_release import setup_hook

    setup_hook(sys.argv)
except ImportError:
    pass
